/*****************************************************
* Chad's OpenGL light casting program                *
* Rotating light source around a Torus in 3D space   *
* Created 12-17-2008                                 *
*                                                    *
*****************************************************/

#include <GL/glut.h>
#include <stdlib.h>
#include <stdio.h>
#include "math.h"
static int spinX, spinY = 0;

// GLfloat cameraX, cameraY, cameraZ;
/*  Initialize material property, light source, lighting model,
 *  and depth buffer.
 */

void init(void) 
{
   GLfloat mat_specular[] = { 1.0, 1.0, 1.0, 1.0 };
   GLfloat mat_shininess[] = { 50.0 };
   GLfloat light_position[] = { 1.0, 1.0, 1.0, 0.0 };
   /*GLfloat light_position2[] = { -2.0, 1.5, 2.0, 0.0 };*/
   GLfloat mat_diffuse[] = { .5, 1.0, .5, 0.0 };
   /*GLfloat blue_light[] = { 0.0, 0.0, 2.0, 1.0 };
   GLfloat red_light[] = { 2.0, 0.0, 0.0, 1.0 };*/

   glClearColor (0.0, 0.0, 0.2, 0.0);
   glShadeModel (GL_SMOOTH);

   glMaterialfv(GL_FRONT, GL_SPECULAR, mat_specular);
   glMaterialfv(GL_FRONT, GL_SHININESS, mat_shininess);
   glMaterialfv(GL_FRONT, GL_DIFFUSE, mat_diffuse);

   glLightfv(GL_LIGHT0, GL_POSITION, light_position);
   /*glLightfv(GL_LIGHT0, GL_SPECULAR, blue_light);
   glLightfv(GL_LIGHT1, GL_POSITION, light_position2);
   glLightfv(GL_LIGHT1, GL_SPECULAR, red_light);*/

   glEnable(GL_LIGHTING);
   glEnable(GL_LIGHT0);
   //glEnable(GL_LIGHT1);
   glEnable(GL_DEPTH_TEST);
   glMatrixMode(GL_PROJECTION);
   
}

void display(void)
{
   GLfloat position[] = { 0.0, 0.0, 1.5, 1.0 };

   glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
   glPushMatrix ();
   gluLookAt (0.0, 0.0, 5.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0);
   
   glPushMatrix ();
   glRotated ((GLdouble) spinY, 1.0, 0.0, 0.0);
   glLightfv (GL_LIGHT0, GL_POSITION, position);

   glRotated ((GLdouble) spinX, 0.0, 1.0, 0.0);
   glLightfv (GL_LIGHT0, GL_POSITION, position);

   glTranslated (0.0, 0.0, 1.5);
   glDisable (GL_LIGHTING);
   glColor3f (0.0, 1.0, 1.0);
   glutWireCube (0.1);
   glEnable (GL_LIGHTING);
   glPopMatrix ();

   //glPushMatrix();
   //glTranslatef(-.75, 0.5, 0.0);
   //glRotatef(90.0, 1.0, 1.0, 0.0);
   glutSolidTorus (0.275, 0.85, 100, 150);
   glPopMatrix ();
   glFlush ();
}

void reshape (int w, int h)
{
   glViewport (0, 0, (GLsizei) w, (GLsizei) h);
   glMatrixMode (GL_PROJECTION);
   glLoadIdentity();
   if (w <= h)
      glOrtho (-1.5, 1.5, -1.5*(GLfloat)h/(GLfloat)w,
         1.5*(GLfloat)h/(GLfloat)w, -10.0, 10.0);
   else
      glOrtho (-1.5*(GLfloat)w/(GLfloat)h,
         1.5*(GLfloat)w/(GLfloat)h, -1.5, 1.5, -10.0, 10.0);
   glMatrixMode(GL_MODELVIEW);
   glLoadIdentity();
}

void keyboard(unsigned char key, int x, int y)
{
    switch (key) {
      case 'X':
         spinX = (spinX + 5) % 360;
         glutPostRedisplay();
         default:
         break;  
      case 'x':
         spinX = (spinX - 5) % 360;
         glutPostRedisplay();
         break; 
      case 'Y':
         spinY = (spinY + 5) % 360;
         glutPostRedisplay();
         break;  
      case 'y':
         spinY = (spinY - 5) % 360;
         glutPostRedisplay();
         break; 
      /*case 27:
         exit(0);
         break; */
   }
}

int main(int argc, char** argv)
{
   printf( "Welcome to Chad's Light Casting program. version 1.0 \n");
   printf( "\n");
   printf( "This program demonstrates the use of an OpenGL lighting model.\n");
   printf( "A torus is drawn using a green material characteristic.\n");
   printf( "\n");
   printf( "A single light source illuminates the object\n");
   printf( "and the user can rotate the light source in positive or negative\n");
   printf( "X and Y directions\n");
   printf( "\n");
   printf( "Either tap or hold the X and Y keys to rotate the light source\n");
   printf( "around the torus.\n");
   printf( "Holding 'Shift' + 'X' will move the light source in the positive X direction. \n");
   printf( "Holding 'x' will move the light source in the negative X direction. \n");
   printf( "Holding 'Shift' + 'Y' will move the light source in the positive Y direction. \n");
   printf( "Holding 'y' will move the light source in the negative Y direction. \n");
   
   /*printf("Input the location for 'X' (default 0): ");
   scanf ("%f", &cameraX);
   printf("Input the location for 'Y' (default 0): ");
   scanf ("%f", &cameraY);
   printf("Input the location for 'Z' (default 5): ");
   scanf ("%f", &cameraZ);*/
   glutInit(&argc, argv);
   glutInitDisplayMode (GLUT_SINGLE | GLUT_RGB | GLUT_DEPTH);
   glutInitWindowSize (500, 500); 
   glutInitWindowPosition (750, 100);
   glutCreateWindow("Light Casting");
   init ();   
   glutDisplayFunc(display); 
   glutReshapeFunc(reshape);
   glutKeyboardFunc(keyboard);
   glutMainLoop();
   return 0;
}
